;  ============ NSM lab ===========
;Reference pre-Amplifier PA-8000 (8MHz version)

;#DEFINE DEBUG

#INCLUDE "LIBREG.ASM"	;8051 SFR set
#INCLUDE "LIBMAC.ASM"	;macros

#DEFINE CLK_8MHZ		;RC-5 decoder clock
CLK_KHZ	= 8000		;OSC frequency, KHZ
RTC_MS	= 10		;System clock , MS

#INCLUDE "LIBRTC.ASM"
;This library calculates system timer values
;RTCVH and RTCVL for CLK_KHZ clock frequency
;and RTC_MS system clock period.

#IFDEF DEBUG
#INCLUDE "LIBDEF.ASM"
#ENDIF

;  ------Constantes------

STACK	.EQU 052H		;stack load value
SYS	.EQU 00AH		;RC-5 system
W8582	.EQU 0A0H		;address PCF8582 (A0,A1,A2=0) and write mode
R8582	.EQU 0A1H		;address PCF8582 (A0,A1,A2=0) and read  mode
LEVEL	.EQU 010H		;AIDENT and STANDBY level
VIDEL	.EQU 8	   	;Volume Control Initial Delay (60mS)
VDEL	.EQU 16   	;Volume Control Delay (160mS)
DCLDEL	.EQU 40		;Double-Click Delay (400mS)
MDEL	.EQU 50   	;Mute Delay (500mS)
MDEL1	.EQU 30   	;Mute Delay1 (300mS)
IDEL	.EQU 50   	;Input Switch Delay (500mS)
IDEL1	.EQU 100  	;Motor Input Switch Delay (1000mS)
REVD	.EQU 20   	;Reverse delay (200mS)
SDEL	.EQU 200  	;Switcher Motor Delay (2S)
ADEL	.EQU 100  	;Auto Ident Mode Delay (1S)
FDEL	.EQU 135  	;Off Delay (135x2,56S)

;------Ports------

COM	.EQU P1.0		;ADC comparator
STB	.EQU P1.1		;standby mode
UPD	.EQU P1.2		;up/down state of input switcher
SOUND	.EQU P1.3		;sound enable line
LIS	.EQU P1.4		;return line of listen switcher
REC	.EQU P1.5		;return line of record switcher
RET0	.EQU P1.6		;keyboard return line 0
RET1	.EQU P1.7		;keyboard return line 1
SDA	.EQU RXD		;I2C-bus data
SCL	.EQU TXD		;I2C-bus clock
SER	.EQU INT0		;remote control RC5 code
KBD	.EQU INT1		;keyboard interrupt
TE	.EQU T0		;external bus enable
EZ	.EQU T1		;Z-state enable

;------Bit Addressing Memory------

MEMSLR	 .EQU 020H	;REGSLR copy
SCAN0	 .EQU M_20H.0	;keyboard scan line  0
SCAN1	 .EQU M_20H.1	;keyboard scan line  1
LIS0	 .EQU M_20H.2	;listen switcher bit 0
LIS1	 .EQU M_20H.3	;listen switcher bit 1
LIS2	 .EQU M_20H.4	;listen switcher bit 2
REC0	 .EQU M_20H.5	;record switcher bit 0
REC1	 .EQU M_20H.6	;record switcher bit 1
REC2	 .EQU M_20H.7	;record switcher bit 2

MEMSUD	 .EQU 021H	;REGSUD copy
M20REL	 .EQU M_21H.0	;MUTE -20dB relay
SA0	 .EQU M_21H.1	;spectrum analyzer channel bit 0
SA1	 .EQU M_21H.2	;spectrum analyzer channel bit 1
SA2	 .EQU M_21H.3	;spectrum analyzer channel bit 2
UP2	 .EQU M_21H.4	;input switcher motor up
DN2	 .EQU M_21H.5	;input switcher motor down
UP1	 .EQU M_21H.6	;volume controller motor up
DN1	 .EQU M_21H.7	;volume controller motor down

MEMEQR	 .EQU 022H	;REGEQR copy
RISD	 .EQU M_22H.0	;Record Input Selector Enable
LISD	 .EQU M_22H.1	;Listen Input Selector Enable
LEDBPS	 .EQU M_22H.2	;led EQ. BYPASS
MREL	 .EQU M_22H.3	;MUTE relay
CREL	 .EQU M_22H.4	;MODE3 relay
BREL	 .EQU M_22H.5	;MODE2 relay
AREL	 .EQU M_22H.6	;MODE1 relay
EREL	 .EQU M_22H.7	;EQ. BYPASS relay

MEMLSR	 .EQU 023H	;REGLSR copy
RELM1	 .EQU M_23H.0	;relay monitor 1
RELM2	 .EQU M_23H.1	;relay monitor 2
RELR0	 .EQU M_23H.2	;relay record bit 0
RELR1	 .EQU M_23H.3	;relay record bit 1
RELR2	 .EQU M_23H.4	;relay record bit 2
RELL0	 .EQU M_23H.5	;relay listen bit 0
RELL1	 .EQU M_23H.6	;relay listen bit 1
RELL2	 .EQU M_23H.7	;relay listen bit 2

MEMLED	 .EQU 024H	;LEDSR copy
SCALE	 .EQU M_24H.0	;led display scale
LEDDAT	 .EQU M_24H.1	;led DAT
LEDSWP	 .EQU M_24H.2	;led SW
LEDRGT	 .EQU M_24H.3	;led R
LEDLFT	 .EQU M_24H.4	;led L
LEDMON	 .EQU M_24H.5	;led MONO
LEDREV	 .EQU M_24H.6	;led REV
LEDMUT	 .EQU M_24H.7	;led MUTE

BMEM1	 .EQU 025H
GMB	 .EQU M_25H.0	;global mute bit
CMB	 .EQU M_25H.1	;control mute bit
DEN	 .EQU M_25H.2	;display enable bit
PRESS	 .EQU M_25H.3	;local keyboard new press flag
VCB	 .EQU M_25H.4	;volume control bit
MTB	 .EQU M_25H.5	;mute timer bit
WTB	 .EQU M_25H.6	;switcher timer bit
SMTB	 .EQU M_25H.7	;switcher motor timer bit

RTPC	 .EQU 026H
T20M	 .EQU M_26H.0	;20mS period bit
T40M	 .EQU M_26H.1	;40mS period bit
T80M	 .EQU M_26H.2	;80mS period bit
T160M	 .EQU M_26H.3	;160mS period bit
T320M	 .EQU M_26H.4	;320mS period bit
T640M	 .EQU M_26H.5	;640mS period bit
T1S28	 .EQU M_26H.6	;1.28S period bit
T2S56	 .EQU M_26H.7	;2.56S period bit

BMEM2	 .EQU 027H
SWB	 .EQU M_27H.0	;swap bit
SFPB	 .EQU M_27H.1	;standby function processing bit
AIDB	 .EQU M_27H.2	;auto ident bit
ATB	 .EQU M_27H.3	;autoident timer bit
ASBB	 .EQU M_27H.4	;autostandby bit
HP	 .EQU M_27H.5	;autostandby half period bit
SWPE	 .EQU M_27H.6	;swap led enable bit
MREV	 .EQU M_27H.7	;motor reverse bit

BMEM3	 .EQU 028H
CM20B	 .EQU M_28H.0	;control mute20 bit
DCLB	 .EQU M_28H.1	;double-click bit

;------Internal Memory------

SACH	.EQU 030H 	;Spectrum Analyzer Channel
MODN	.EQU 031H 	;Mode Number
CODE	.EQU 032H 	;Control Codes Store
CSWL	.EQU 033H 	;Current Listen Switcher Position
CSWR	.EQU 034H 	;Current Record Switcher Position
CLIS	.EQU 035H 	;Current Listen Input
CREC	.EQU 036H 	;Current Record Input
MISIN	.EQU 037H 	;Missing listen Input number
VOLTM	.EQU 038H 	;Volume Control Timer
MUTTM	.EQU 039H 	;Mute Timer
SWTM	.EQU 03AH 	;Switcher Timer
SWMTM	.EQU 03BH 	;Switcher Motor Timer
AIDTM	.EQU 03CH 	;Auto Ident Timer
MAXL	.EQU 03DH 	;Maximum Level
MNA	.EQU 03EH 	;Max number of aident cycles
STBTM	.EQU 03FH 	;Standby Timer
SSWL	.EQU 040H 	;Swap Listen Switcher Position
DCLTM	.EQU 041H 	;Double-Click Timer
SWAPT	.EQU 042H 	;Swap Table (8 bytes)

;------External Memory------

SPAN	.EQU 000H 	;spectrum analyzer base address
SPAN1	.EQU 004H 	; 50 Hz  spectrum analyzer LEDs register
SPAN2	.EQU 005H 	;125 Hz  spectrum analyzer LEDs register
SPAN3	.EQU 006H 	;400 Hz  spectrum analyzer LEDs register
SPAN4	.EQU 007H 	;  1 KHz spectrum analyzer LEDs register
SPAN5	.EQU 000H 	;  3 KHz spectrum analyzer LEDs register
SPAN6	.EQU 001H 	;  7 KHz spectrum analyzer LEDs register
SPAN7	.EQU 002H 	; 16 KHz spectrum analyzer LEDs register
LEDSR	.EQU 003H 	;special LEDs register
REGSLR	.EQU 008H 	;register S0-S1, L0-L2, R0-R2
REGSUD	.EQU 009H 	;register SA0-SA2, U1/D1, U2/D2
DAC	.EQU 00AH 	;DAC register
REGEQR	.EQU 00BH 	;register EQBPL, RELAYS
REGLSR	.EQU 00CH 	;register MON1, MON2, REC, LIS

;------Vectors Area------

	.ORG 0000H 	;reset vector
	LJMP INIT
	.ORG 0003H 	;INT0 vector
	LJMP RC5
	.ORG 000BH 	;INT TIMER 0 vector
	LJMP RTC

INIT:	MOV SP,#STACK	;stack init
	CLR SOUND		;clear sound bit
	CLR STB		;standby off
	LCALL DE100	;delay for power up
	CLR TE		;bus drivers enable
	LCALL CLEAR	;clear registers
	CLR EZ		;registers output enable
	MOV CODE,#0H	;set unused code
	MOV R37,CODE	;same code condition
	SETB PRESS	;set local keyboard new press flag
	MOV SACH,#7H	;spectrum analyzer channel init
	MOV VOLTM,#0H	;clear volume control timer
	CLR VCB		;clear volume control bit
	MOV SWTM,#0H	;clear switcher timer
	CLR WTB		;clear switcher timer bit
	MOV SWMTM,#0H	;clear switcher motor timer
	CLR SMTB		;clear switcher motor timer bit
	MOV MUTTM,#MDEL	;load mute timer
	MOV DCLTM,#0H	;clear double-click timer
	CLR DCLB		;clear double-click bit
	SETB MTB		;set mute timer bit
	SETB GMB		;set global mute bit
	CLR CMB		;clear control mute bit
	CLR CM20B		;clear control mute20 bit
	CLR MREV		;clear motor reverse bit
	SETB LEDMUT	;mute led off
	SETB DEN		;display enable
	CLR SCALE		;display scale on
	CLR SWB		;clear swap bit
	CLR SWPE		;clear swap led enable bit
	CLR SFPB		;clear standby function processing bit
	CLR AIDB		;clear auto ident bit
	LCALL SET		;set NV-Memory parameters
	MOV CLIS,#09H	;all listen inputs are off
	MOV CREC,#09H	;all record inputs are off
	MOV CSWL,#00H
	MOV CSWR,#00H
	LCALL SWN		;switcher normalisation
	LCALL SWI		;switch inputs
	MOV TMOD,#11H	;timer1 and timer0 init
	CLR TR0		;timer 0 stop
	MOV TH0,#RTCVH	;timer 0 load
	MOV TL0,#RTCVL
	SETB TR0		;timer 0 start
	SETB EX0		;enable INT0
	SETB PX0		;high   INT0 priority
	CLR IT0		;level  INT0 activating
	CLR PT0		;low timer0 int priority
	SETB ET0		;timer 0 int enable
	SETB EA		;interrupts enable

;Switcher test:

MAIN:	MOV C,LIS		;local switcher test
	ORL C,REC
	JNC LS1		;jump if no changes
	LCALL SWR		;read switchers
	SETB WTB		;set Switcher Timer Bit
	MOV SWTM,#IDEL	;load delay
	JNB SMTB,LS1
	MOV SWTM,#IDEL1	;load delay 1

;Local keyboard test:

LS1:	MOV C,RET0	;local keyboard test
	ANL C,RET1
	JNC MA3		;jump if press
	SETB PRESS
	SJMP MA2		;jump if no press

MA3:	LCALL SCAN
	MOV R2,A		;save keyboard code
	LCALL DEL10
	LCALL SCAN
	CJNE A,R02,MA2	;no press if codes are differ

	CLR EX0		;RC int disable
	ANL R37,#0C0H	;clear com. bits
	ORL A,R37		;set com. bits
	JNB PRESS,MA4
	CPL ACC.6		;if PRESS = 1 then cpl bit 6
	CLR PRESS		;clear PRESS
MA4:	MOV R37,A		;R37 load, end keyboard holding

;RC test:

MA2:	CLR EX0		;RC int disable
	MOV A,R37		;command code load
	ANL R37,#0C0H	;clear com. bits
	SETB EX0		;RC int enable
	MOV R0,A
	ANL A,#3FH
	JNZ KEYPR
	LJMP MA1		;no press or unused code
KEYPR:	MOV A,R0
	CJNE A,CODE,DIF	;jump if new command
	ANL A,#3FH	;control bits clear
	MOV R7,A
	SJMP SAME

DIF:	MOV CODE,A	;new press, store code
	ANL A,#3FH	;control bits clear
	MOV R7,A

;Control Functions Analysis and Processing:

	JNB STB,MA81           ;skip if standby
	LCALL WAKEUP
	LJMP MAIN
MA81:	JB ACC.4,VCON	   ;jump if volume control
	SETB SOUND             ;beep
	CLR SOUND
	CJNE R7,#CODSTB,$+6H   ;standby code
	LCALL STBY
	CJNE R7,#CODMUT,$+6H   ;mute code
	LCALL MUTE
	CJNE R7,#CODMOD,$+6H   ;mode code
	LCALL MODE
	CJNE R7,#CODEQU,$+6H   ;eq. code
	LCALL EQBP
	CJNE R7,#CODDIS,$+6H   ;display off code
	LCALL DOFF
	CJNE R7,#CODSWP,$+6H   ;swap code
	LCALL SWAP
	CJNE R7,#CODAID,$+6H   ;auto ident code
	LCALL AIDEN
	CJNE R7,#CODASB,$+6H   ;auto standby code
	LCALL ASB
	CJNE R7,#CODTCH,$+6H   ;teach code
	LCALL TEACH
	CJNE R7,#CODUSW,$+6H   ;unswap code
	LCALL UNSWP
	CJNE R7,#CODDAT,$+6H   ;dat input code
	LCALL DATIN
	CJNE R7,#CODMIC+1,$+3H
	JNC VCON               ;jump if code >27H
	CJNE R7,#CODDAT+1,$+3H
	JC VCON                ;jump if code <21H
	LCALL INPUT            ;input switch codes (21H..27H)
VCON:	CJNE R7,#CODVLP,$+6H   ;volume plus
	LCALL VOLP
	CJNE R7,#CODVLM,$+6H   ;volume minus
	LCALL VOLM
SAME:	CJNE R7,#CODVLP,$+6H   ;volume plus
	LCALL VOLPH
	CJNE R7,#CODVLM,$+6H   ;volume minus
	LCALL VOLMH

;Standby mode check:

MA1:	JNB STB,MA11
	LJMP MA20		;loop if standby

;Spectrum analyzer indicate:

MA11:	LCALL SA		;spectrum analyzer indicate

;Volume motor stop:

	JNB VCB,MA5	;jump if no Volume Control Bit
	MOV A,VOLTM
	JNZ MA5		;jump if no Volume Timer overflow
	LCALL VOLS	;volume controller stop

;Mute control:

MA5:	JNB MTB,MA6	;jump if no Mute Timer Bit
	MOV A,MUTTM
	JNZ MA6		;jump if no Mute Timer overflow
	CLR GMB
	LCALL SETMD	;mute off
	CLR MTB		;clear Mute Timer Bit

;Mute double-click processing:

MA6:	JNB DCLB,MA30	;jump if no Double-Click Bit
	MOV A,DCLTM
	JNZ MA30
	CLR CMB
	LCALL SETMD	;switch mode
	CLR DCLB		;clear Double-Click Bit

;Switcher motor control:

MA30:	JNB WTB,MA7	;jump if no Switcher Timer Bit
	MOV A,SWTM
	JNZ MA7		;jump if no Switcher Timer overflow
	LCALL SWI		;input on
	CLR WTB		;clear Switcher Timer Bit
MA7:	JNB SMTB,MA8	;jump if no Switcher Motor Timer Bit
	MOV A,CSWL
	CJNE A,MISIN,MA71	;jump if missing input not reached
	JNB MREV,MA72	;jump to wait for normal position
	MOV A,SWMTM
	JNZ MA8		;jump if no reverse timer overflow
	CLR MREV
	LCALL MOST	;motor stop
	CLR SMTB
	SJMP MA8
MA72:	JB UPD,MA71	;jump if no motor normal position
	CPL UP2
	CPL DN2
	LCALL SETMD	;motor reverse
	MOV SWMTM,#REVD	;load reverse delay
	SETB MREV		;set reverse flag
	SJMP MA8
MA71:	MOV A,SWMTM
	JNZ MA8		;jump if no Switcher Motor Timer overflow
	LCALL MOST	;motor stop
	CLR SMTB
	LCALL EBELL	;Error Bell

;Blink control:

MA8:	MOV C,SWB
	ANL C,T320M	;320mS period bit
	JB SWB,MA97
	ORL C,SWPE
MA97:	CPL C
	MOV LEDSWP,C

	MOV C,CM20B
	ANL C,T640M
	ORL C,CMB
	CPL C
	MOV LEDMUT,C

	MOV R0,#LEDSR
	MOV A,MEMLED
	JNB DEN,MA99
	MOVX @R0,A	;swap led blink

;Long functions processing:

MA99:	JNB SFPB,MA9	;jump if no SFPB bit
	LCALL STBP	;standby function processing
MA9:	JNB AIDB,MA10	;jump if no auto ident mode
	LCALL AIDFP	;auto ident function procession
MA10:	JNB ASBB,MA20
	LCALL ASBFP
MA20:	LJMP MAIN

;------Subroutines Area------

;STANDBY function

STBY:	SETB SFPB		;set Standby Function Processing Bit
	CLR AIDB		;clear auto ident bit
	RET

;Standby function Processing

STBP:	JB VCB,STB1	;check up timer control functions
	JB MTB,STB1
	JB WTB,STB1
	JB SMTB,STB1
	CLR SFPB		;clear Standby Function Processing Bit
	SETB GMB		;set Global Mute Bit
	LCALL CLEAR	;clear registers
	LCALL DEL50
	MOV CLIS,#09H	;all listen inputs are off
	MOV CREC,#09H	;all record inputs are off
	SETB EZ		;registers output disable
	SETB TE		;bus drivers disable
	SETB STB		;power off

STB2:	MOV C,RET0	;local keyboard test
	ANL C,RET1
	JNC STB2		;jump if press
	LCALL DEL50
	MOV C,RET0	;local keyboard test
	ANL C,RET1
	JNC STB2		;jump if bounce

STB1:	RET

;WAKEUP function

WAKEUP:	SETB SOUND	;beep
	CLR SOUND
	CLR STB		;standby off
	LCALL DE100	;delay for power up
	CLR TE		;bus drivers enable
	LCALL CLEAR	;clear registers
	CLR EZ		;registers output enable
	LCALL DE100
	LCALL SETMD	;registers load
	LCALL SWN		;read switchers
	LCALL SWI		;switch input
	RET

;MUTE function

MUTE:     MOV A,DCLTM
	JNZ MDCL		;jump if double-click
	MOV DCLTM,#DCLDEL	;double-click timer load
	CPL CM20B
	JB CM20B,MUTE1
	SETB DCLB		;set double-click bit
	MOV STBTM,#FDEL	;load standby timer
	RET

MDCL:     CLR DCLB
	CPL CMB
	MOV C,CMB
	MOV CM20B,C

MUTE1:	LCALL SETMD
	MOV STBTM,#FDEL	;load standby timer
	RET

;MODE function

MODE:	MOV A,MODN
	INC A
	CJNE A,#5H,STMD
	CLR A
STMD:	MOV MODN,A
	MOV R1,#MDA	;load EEPROM mode address
	LCALL SAVE	;save mode
	LCALL MDBS	;set mode bits
	LCALL SETMD	;mode on
	MOV STBTM,#FDEL	;load standby timer
	RET

;EQ. BYPASS function

EQBP:   	CPL EREL
	MOV C,EREL
	MOV LEDBPS,C
	CLR A
	MOV ACC.0,C
	MOV C,ASBB
	MOV ACC.1,C
	MOV R1,#EQA	;load EEPROM eq. address
	LCALL SAVE	;save eq.
	LCALL SETMD	;switch mode
	MOV STBTM,#FDEL	;load standby timer
	RET

;DISPLAY OFF function

DOFF:	CPL DEN		;set/clear Display Enable Bit
	LCALL SETMD
	MOV STBTM,#FDEL	;load standby timer
	RET

;SWAP function

SWAP:	CPL SWB		;set/clear Swap Bit
	MOV SSWL,#0H	;save DAT swap position
	JNB LEDDAT,SWA1
	MOV SSWL,CSWL	;save swap position
SWA1:	MOV STBTM,#FDEL	;load standby timer
	RET

;AIDENT function

AIDEN:	SETB AIDB		;set auto ident bit
	CLR ATB		;clear auto ident timer bit
	MOV MNA,#32H	;load max number of aident cycles
	MOV STBTM,#FDEL	;load standby timer
	RET

;Auto Ident Function procession

AIDFP:	JB ATB,AID1	;jump if auto ident timer runs
	MOV MAXL,#0H	;clear max level
	MOV AIDTM,#ADEL	;auto ident timer load
	SETB ATB		;set auto ident timer bit
	RET
AID1:	JNB GMB,AID6
	MOV MAXL,#0H	;clear max level if mute
AID6:	MOV A,AIDTM
	JZ AID2		;jump if timer overflow
	RET
AID2:	CLR ATB
	MOV A,MAXL
	CJNE A,#LEVEL,$+3H	;ident level check
	JNC AID3		;jump if end of AIDENT mode
	DEC MNA
	MOV A,MNA
	JNZ AID4
	RET		;ret if number of cycles is over
AID4:	INC CSWL		;next in
	ANL CSWL,#7H
	LCALL SWI		;switch in
	RET
AID3:	CLR AIDB		;end of AIDENT mode
	MOV A,CSWL
	JZ AID5
	ADD A,#20H
	MOV R7,A
	LCALL SWR
	LCALL INPUT	;input on
	RET
AID5:	ADD A,#20H
	MOV R7,A
	LCALL DATIN
	RET

;AUTOSTANDBY function

ASB:	CPL ASBB		;set auto standby bit
	CLR A
	MOV C,EREL
	MOV ACC.0,C
	MOV C,ASBB
	MOV ACC.1,C
	MOV R1,#EQA
	LCALL SAVE
	JNB ASBB,ASB1	;jump if mode off
	MOV STBTM,#FDEL	;load standby timer
	CLR HP
	RET
ASB1:	LCALL DE100
	SETB SOUND	;double beep when off mode
	CLR SOUND
	RET

;Auto Standby function processing

ASBFP:	JB HP,ASB2
	JNB T2S56,ASB3
	MOV A,MAXL
	MOV MAXL,#0H
	DEC STBTM
	CJNE A,#LEVEL,$+3H
	JC ASB4		;jump if no level
	MOV STBTM,#FDEL	;level O.K., timer reload
	SETB HP
	RET
ASB4:	MOV A,STBTM
	JZ ASB5		;jump if timer is over
	SETB HP
	RET
ASB2:	JB T2S56,ASB3
	CLR HP
ASB3:	RET
ASB5:	MOV STBTM,#FDEL	;timer reload
	LCALL STBY	;standby
	RET

;TEACH function

TEACH:    MOV STBTM,#FDEL	;load standby timer
	RET

;UNSWAP function

UNSWP:	CLR SWB		;NV-Memory and Swap Table init

	MOV R1,#MDA
	CLR A
	MOV MODN,A
	LCALL SAVE	;clear mode (MODE=0)
	LCALL MDBS	;set mode bits

	MOV R1,#EQA
	MOV A,#03H
	LCALL SAVE	;clear EQ. and ASBB (EQ.=1, ASBB=1)
	SETB LEDBPS
	SETB EREL
	SETB ASBB
	MOV STBTM,#FDEL	;load standby timer
	CLR HP
	LCALL SETMD	;switch mode

	CLR A
	MOV R0,#SWAPT	;swap table address in RAM
	MOV R1,#STA	;swap table address in EEPROM
	MOV R2,#08H	;number of bytes
UN1:	MOV @R0,A
	LCALL SAVE	;clear swap table (TABLE=0,1,2...)
	INC A
	INC R0
	INC R1
	DJNZ R2,UN1

	LCALL SWI		;switch input
	RET

;DAT INPUT function

DATIN:	CLR AIDB		;clear auto ident bit
	MOV CSWL,#0H
	LCALL SWI
	LCALL SWR		;restore CSWL
	MOV STBTM,#FDEL	;load standby timer
	RET

;INPUT switch function

INPUT:	MOV STBTM,#FDEL	;load standby timer
	MOV A,R7
	CLR C
	SUBB A,#20H	;A=1..7
	MOV MISIN,A	;save input number
	CJNE A,CSWL,INP1
	JB LEDDAT,INP2
	LCALL SWI
INP2:	RET		;ret if input allready on
INP1:	MOV DN2,C
	CPL C
	MOV UP2,C
	SETB SMTB		;set Switcher Motor Timer Bit
	MOV SWMTM,#SDEL	;load timer
	LCALL SETMD	;motor on
	RET

;Switcher Motor Stop function

MOST:	CLR UP2
	CLR DN2
	LCALL SETMD
	RET

;VOLUME PLUS function

VOLP:	MOV VOLTM,#VIDEL	;Volume Control Timer load
	MOV R7,#NONE
	SJMP DVP
VOLPH:	MOV VOLTM,#VDEL	;Volume Control Timer load
DVP:	SETB VCB		;set Volume Control Bit
	SETB UP1
	CLR DN1
	LCALL SETMD	;motor on
	RET

;VOLUME MINUS function

VOLM:	MOV VOLTM,#VIDEL	;Volume Control Timer load
	MOV R7,#NONE
	SJMP DVM
VOLMH:	MOV VOLTM,#VDEL	;Volume Control Timer load
DVM:	SETB VCB		;set Volume Control Bit
	SETB DN1
	CLR UP1
	LCALL SETMD	;motor on
	RET

;VOLUME STOP function

VOLS:	CLR VCB		;clear Volume Control Bit
	CLR UP1
	CLR DN1
	LCALL SETMD	;motor stop
	RET

;Clear Registers

CLEAR:	MOV R1,#08H	;registers counter load
	MOV R0,#SPAN	;initial registers address load
	MOV A,#0FFH	;load value
CL1:	MOVX @R0,A
	INC R0
	DJNZ R1,CL1
	MOV A,#00H
	MOV MEMSLR,A	;copy init
	MOV R0,#REGSLR
	MOVX @R0,A	;REGSLR clear
	MOV A,#02H
	MOV MEMSUD,A	;copy init
	MOV R0,#REGSUD
	MOVX @R0,A	;REGSUD clear
	MOV A,#00H
	MOV R0,#DAC
	MOVX @R0,A	;DAC register clear
	MOV A,#0FFH
	MOV MEMEQR,A	;copy init
	MOV R0,#REGEQR
	MOVX @R0,A	;REGEQR clear
	MOV A,#03H
	MOV MEMLSR,A	;copy init
	MOV R0,#REGLSR
	MOVX @R0,A	;REGLSR clear
	RET

;Set mode bits

MDBS:	JZ MD0		;sets bits AREL,BREL,CREL and leds bit
	DEC A		;A - mode number
	JZ MD1
	DEC A
	JZ MD2
	DEC A
	JZ MD3
	SETB AREL		;mode4 set (R+L)
	SETB BREL
	CLR CREL
	SETB LEDREV
	CLR LEDMON
	CLR LEDLFT
	CLR LEDRGT
	SJMP MD4
MD0: 	SETB AREL		;mode0 set (NORM)
	SETB BREL
	SETB CREL
	SETB LEDREV
	SETB LEDMON
	SETB LEDLFT
	SETB LEDRGT
	SJMP MD4
MD1: 	CLR AREL		;mode1 set (REV)
	CLR BREL
	SETB CREL
	CLR LEDREV
	SETB LEDMON
	SETB LEDLFT
	SETB LEDRGT
	SJMP MD4
MD2: 	SETB AREL		;mode2 set (L)
	CLR BREL
	SETB CREL
	SETB LEDREV
	CLR LEDMON
	CLR LEDLFT
	SETB LEDRGT
	SJMP MD4
MD3: 	CLR AREL		;mode3 set (R)
	SETB BREL
	SETB CREL
	SETB LEDREV
	CLR LEDMON
	SETB LEDLFT
	CLR LEDRGT
MD4:	RET

;Set NVM parameters

SET:	MOV R1,#MDA	;load NV-Memory mode address
	LCALL READ	;read mode number
	CJNE A,#05H,$+3H
	JC SET2
	CLR A		;clear A if A>4
SET2:	MOV MODN,A	;save mode number
	LCALL MDBS	;set mode bits
	MOV R1,#EQA	;load NV-Memory eq. address
	LCALL READ	;read eq. byte
	MOV C,ACC.0	;set eq. bits
	MOV EREL,C
	MOV LEDBPS,C
	MOV C,ACC.1
	MOV ASBB,C	;set auto standby enable bit
	JNC SET3		;jump if mode off
	MOV STBTM,#FDEL	;load standby timer
	CLR HP
SET3:	LCALL SETMD	;switch mode
	MOV R1,#STA
	MOV R2,#08H
	MOV R0,#SWAPT
SET1:	LCALL READ	;swap table read
	ANL A,#07H
	MOV @R0,A
	INC R0
	INC R1
	DJNZ R2,SET1
	RET

;Switchers Read

SWR:	CLR AIDB		;clear auto ident bit

	MOV R0,#REGSLR	;REGSLR address load
	MOV R1,#1H	;code init
	MOV R2,#8H
SWR2:	MOV A,R1
	DEC A
	RL A
	RL A
	ANL MEMSLR,#0E3H
	ORL A,MEMSLR
	MOV MEMSLR,A
	MOV R3,A
	MOVX @R0,A
	NOP
	JNB LIS,SWR1
	INC R1
	DJNZ R2,SWR2
	SJMP SWR5		;jump if abnormal exit (CSWL not changes)

SWR1:	MOV CSWL,R1	;save Listen Input code

	JNB SMTB,SWR5	;jump if no Switcher Motor Timer Bit
	MOV A,MISIN
	CJNE A,CSWL,SWR88
	SJMP SWR5

SWR88:	MOV DN2,C
	CPL C
	MOV UP2,C
	MOV A,MEMSUD
	MOV R0,#REGSUD
	MOVX @R0,A	;motor dir correction

	MOV SWMTM,#SDEL	;reload Switcher Motor Timer

SWR5:	MOV R0,#REGSLR	;REGSLR address load
	MOV R1,#1H	;code init
	MOV R2,#8H
SWR4:	MOV A,R1
	DEC A
	SWAP A
	RL A
	ANL MEMSLR,#01FH
	ORL A,MEMSLR
	MOV MEMSLR,A
	MOV R3,A
	MOVX @R0,A
	NOP
	JNB REC,SWR3
	INC R1
	DJNZ R2,SWR4
	SJMP SWR7		;jump if abnormal exit (CSWR not changes)

SWR3:	MOV CSWR,R1	;save Record Input code

SWR7:	RET

;Switcher Normalisation

SWN:	LCALL SWR		;Read Switchers
	LCALL MOTN	;Motor Normalisation
	MOV A,CSWL
	XRL A,#8H
	JNZ SWN5		;jump if CSWL<>8
	LCALL SWR		;Read Switchers
	RET
SWN5:	MOV R4,CSWL	;CSWL save
	LCALL SWR
	MOV A,R4
	CJNE A,CSWL,SWN6
	RET		;ret if no changes
SWN6:	ADD A,#20H
	MOV R7,A
	LCALL INPUT	;restore input
	RET

;Motor Normalisation

MOTN:	JB UPD,SWN1
	RET
SWN1:	MOV A,CSWL
	CJNE A,#2H,$+3H	;if CSWL>1 then C=0
	MOV UP2,C		;CSWL=1 - UP2=1,DN2=0
	CPL C
	MOV DN2,C		;CSWL>1 - DN2=1,UP2=0
	LCALL SETMD	;motor on
	MOV R4,#200	;load delay 2S
SWN3:	JNB UPD,SWN2	;O.K.
	MOV R2,#20
	LCALL DEL05	;delay 10mS
	DJNZ R4,SWN3
	CPL UP2
	CPL DN2
	LCALL SETMD	;motor reverse
	MOV R4,#200	;load delay 2S
SWN4:	JNB UPD,SWN2	;O.K.
	MOV R2,#20
	LCALL DEL05	;delay 10mS
	DJNZ R4,SWN4
	LCALL EBELL	;error bell
SWN2:	LCALL MOST
	RET

;Switch Inputs

SWI:	SETB LEDDAT
	MOV A,CSWL
	JNZ SWI5
	CLR LEDDAT
SWI5:	MOV R0,#LEDSR
	MOV A,MEMLED
	JNB DEN,SWI11
	MOVX @R0,A	;DAT led on/off

SWI11:	JNB SWB,SWI2	;jump if not swap mode

	MOV A,CSWL
	ADD A,#SWAPT
	MOV R0,A		;swap table address 1 in RAM

	MOV A,@R0		;swap table read
	MOV R2,A		;save code

	MOV A,SSWL
	ADD A,#SWAPT
	MOV R0,A		;swap table address 2 in RAM

	MOV A,SSWL
	ADD A,#STA
	MOV R1,A		;swap table address 2 in NV RAM

	MOV A,R2
	MOV @R0,A		;save 2 to RAM
	LCALL SAVE	;save 2 to NV RAM

	MOV A,CSWL
	ADD A,#SWAPT
	MOV R0,A		;swap table address 1 in RAM

	MOV A,CSWL
	ADD A,#STA
	MOV R1,A		;swap table address 1 in NV RAM

	MOV A,CLIS
	MOV @R0,A		;save 1 to RAM
	LCALL SAVE	;save 1 to NV RAM

	SETB SWPE
	CJNE A,CSWL,SWI3
	CLR SWPE

SWI3:	CLR SWB
	SJMP SWI1

SWI2:	MOV A,CSWL	;not swap mode
	ADD A,#SWAPT
	MOV R0,A		;swap table address in RAM
	MOV A,@R0		;read swap table
	CJNE A,CLIS,SWI4

	SETB SWPE
	CJNE A,CSWL,SWI1
	CLR SWPE
	SJMP SWI1		;jump if same input

SWI4:	MOV CLIS,A
	SETB GMB
	LCALL SETMD	;mute on
	LCALL DEL50	;delay 50mS
	SETB LISD
	LCALL SETMD	;all inputs off
	LCALL DEL50	;delay 50mS

	MOV A,CLIS
	SETB SWPE
	CJNE A,CSWL,SWI6
	CLR SWPE
SWI6:	SWAP A
	RL A
	ANL MEMLSR,#1FH
	ORL A,MEMLSR
	MOV MEMLSR,A
	CLR LISD
	LCALL SETMD	;input on
	SETB MTB
	MOV MUTTM,#MDEL1	;load mute delay

SWI1:	MOV A,CSWR
	ADD A,#SWAPT
	MOV R0,A		;swap table address in RAM
	MOV A,@R0		;read swap table
	CJNE A,CREC,SWI8
	SJMP SWI7		;jump if same input

SWI8:	MOV CREC,A
	SETB RELM1
	SETB RELM2
	LCALL SETMD	;monitors off
	LCALL DEL25	;delay 25mS
	SETB RISD
	LCALL SETMD	;inputs off
	LCALL DEL25	;delay 25mS
	MOV A,CSWR
	CLR RELM1
	CLR RELM2
	CJNE A,#02H,SWI9
	SETB RELM1
SWI9:	CJNE A,#03H,SWI10
	SETB RELM2
SWI10:	MOV A,CREC
	RL A
	RL A
	ANL MEMLSR,#0E3H
	ORL A,MEMLSR
	MOV MEMLSR,A
	CLR RISD
	LCALL SETMD	;input and monitors on
SWI7:	RET

;Spectrum Analyzer ADC Routine

SA:	MOV A,SACH	;load spectrum analyzer channel
	DEC A
	JNZ S1
	MOV A,#07H
S1:	MOV SACH,A	;new channel in A
	RL A
	ANL MEMSUD,#0F1H
	ORL A,MEMSUD
	MOV MEMSUD,A
	MOV R0,#REGSUD
	MOVX @R0,A	;switch channel

	MOV R0,#DAC	;A/D - convertion for left channel
	MOV R1,#08H	;R5 - output 8-bit code
	MOV R2,#01H
	MOV R5,#00H
ADCL1:	MOV A,R2
	RR A
	MOV R2,A
	ORL A,R5
	MOVX @R0,A
	NOP
	NOP
	NOP
	NOP
	JB COM,ADCL2
	MOV R5,A
ADCL2:	DJNZ R1,ADCL1
	CJNE A,MAXL,$+3H
	JC LC
	MOV MAXL,A	;MAX level store

LC:	MOV DPTR,#LTAB	;R5 - 8-bit input code
	MOV R0,#0H	;R2 - output code
	MOV R2,#0H
	MOV R4,#8H
LC1:	MOV A,R0
	MOVC A,@A+DPTR
	MOV R3,A
	MOV A,R5
	CLR C
	SUBB A,R3
	MOV A,R2
	RRC A
	MOV R2,A
	INC R0
	DJNZ R4,LC1

	MOV A,SACH
	MOV DPTR,#SATAB
	MOVC A,@A+DPTR
	MOV R0,A		;register address
	MOV A,#0FFH
	MOV C,GMB
	ORL C,CMB
	JC S2		;leds off if global mute
	JNB DEN,S2	;leds off if display disable
	MOV A,R2
S2:	MOVX @R0,A	;register load

	RET

;Set mode: EQ. leds, RELAYS

SETMD:	MOV C,GMB
	ORL C,CMB		;make mute bit
	JC MU_ON

MU_OFF:   MOV C,CM20B
	MOV M20REL,C
	JNB MREL,SMD1
	SETB M20REL
	CLR MREL
	LCALL REG_LD
	LCALL DEL10
	CLR M20REL
	SJMP SMD1

MU_ON:	JB MREL,SMD1
	SETB M20REL
	LCALL REG_LD
	LCALL DEL10
	SETB MREL

SMD1:	LCALL REG_LD
	RET

;Registers load

REG_LD:	MOV A,MEMEQR	;REGEQR copy load
	MOV R0,#REGEQR
	MOVX @R0,A	;load REGEQR
	MOV A,#0FFH
	JNB DEN,ST1	;jump if display disable
	MOV A,MEMLED	;LEDSR copy load
ST1:	MOV R0,#LEDSR
	MOVX @R0,A	;load LEDSR
	MOV A,MEMSUD
	MOV R0,#REGSUD
	MOVX @R0,A	;load REGSUD
	MOV A,MEMLSR
	MOV R0,#REGLSR
	MOVX @R0,A	;load REGLSR
	RET

;Local Keyboard Scan

SCAN:	JNB STB,SCN2
	LCALL WAKEUP

SCN3:	MOV C,RET0	;local keyboard test
	ANL C,RET1
	JNC SCN3		;jump if press
	LCALL DEL50
	MOV C,RET0	;local keyboard test
	ANL C,RET1
	JNC SCN3		;jump if bounce

	MOV A,#1H
	RET

SCN2:	MOV R0,#REGSLR	;keyboard scan
	MOV R1,#0H	;scan-code init
	SETB SCAN1
	MOV A,MEMSLR
	MOVX @R0,A
	JNB RET0,SCN1	;code=0
	INC R1
	JNB RET1,SCN1	;code=1
	INC R1
	SETB SCAN0
	CLR SCAN1
	MOV A,MEMSLR
	MOVX @R0,A
	JNB RET0,SCN1	;code=2
	INC R1
	JNB RET1,SCN1	;code=3
	INC R1		;code=4, no press
SCN1:	CLR SCAN0
	MOV A,MEMSLR
	MOVX @R0,A
	MOV A,R1
	MOV DPTR,#KEYTAB
	MOVC A,@A+DPTR
	RET

;Error Bell

EBELL:	SETB SOUND
	CLR SOUND
	LCALL DE100
	SETB SOUND
	CLR SOUND
	LCALL DE100
	SETB SOUND
	CLR SOUND
	RET

;Delay subroutines

DEL10:	PUSH R02		;delay 10mS, uses R2, R3
	PUSH R03
	MOV R2,#20
	LCALL DEL05
	POP R03
	POP R02
	RET

DEL25:	PUSH R02		;delay 25mS, uses R2, R3
	PUSH R03
	MOV R2,#50
	LCALL DEL05
	POP R03
	POP R02
	RET

DEL50:	MOV R2,#100	;delay 50mS, uses R2, R3
	LCALL DEL05
	RET

DE100:	MOV R2,#200	;delay 100mS, uses R2, R3
	LCALL DEL05
	RET

DEL05:	MOV R3,#CLK_KHZ/48	;delay R2x0.5mS, uses R2,R3
	DJNZ R3,$
	DJNZ R2,DEL05
	RET

;I2C NVM memory support

;NV - memory address map

MDA	.EQU 000H		;NV-Memory address for mode byte
EQA	.EQU 001H		;NV-Memory address for eq. byte
STA	.EQU 002H		;NV-Memory address for swap table (8 bytes)

;Read byte from NV - memory
;Input: R1 - address
;Out:   A  - data

READ:	PUSH R00		;reads byte from PCF8582 to A, R1 - address
	MOV A,#W8582	;address PCF8582 and mode write
	LCALL I2CTXD	;transmission
	MOV A,R1		;address in PCF8582
	LCALL I2CTXD	;transmission
	LCALL I2CSTOP
	MOV A,#R8582	;address PCF8582 and mode read
	LCALL I2CTXD	;transmission
	LCALL I2CLRXD	;receive byte
	LCALL I2CSTOP
	POP R00
	RET

;Save byte in NV - memory
;Input: R1 - address, A - data

SAVE:	PUSH R00		;saves byte in PCF8582 from A, R1 - address
	PUSH R02
	MOV R2,A
	MOV A,#W8582	;address PCF8582 and mode write
	LCALL I2CTXD	;transmission
	MOV A,R1		;address in PCF8582
	LCALL I2CTXD	;transmission
	MOV A,R2		;load byte
	LCALL I2CTXD	;transmission
	LCALL I2CSTOP
	LCALL DEL25
	MOV A,R2
	POP R02
	POP R00
	RET

;Transmit byte from A to I2C

I2CTXD:  	MOV R0,#9H	;bit counter load
	CLR C
	CPL C		;set bit C for bit9=1 (when ACK)
	LCALL SDA0	;SDA 1/0 - start
TX1:	LCALL SCL0	;SCL 1/0
	RLC A
	JC OUTP1		;jump if bit=1
	LCALL SDA0	;else SDA=0
OUTP0:	LCALL SCL1	;SCL 0/1
	DJNZ R0,TX1
	RET
OUTP1:	LCALL SDA1	;SDA=1 if bit=1
	SJMP OUTP0

;Receive byte from I2C to A

I2CRXD:	MOV A,#1H		;A init for 8 bit
RX1:	LCALL SCL0	;SCL 1/0
	LCALL SDA1	;SDA=1 - it makes SDA line free
	LCALL SCL1	;SCL 0/1
	MOV C,SDA		;bit from SDA to C
	RLC A		;bit from C to A
	JNC RX1
	LCALL SCL0	;SCL 1/0
	LCALL SDA0	;SDA=0 - ACK
	LCALL SCL1	;SCL 0/1
	RET

;Receive byte from I2C to A and no ACK

I2CLRXD:	MOV A,#1H
LRX1:	LCALL SCL0
	LCALL SDA1
	LCALL SCL1
	MOV C,SDA
	RLC A
	JNC LRX1
	LCALL SCL0
	LCALL SCL1	;SCL 0/1 when SDA=1 - no ACK
	RET

;Stop condition generation

I2CSTOP:	LCALL SCL0
	LCALL SDA0
	LCALL SCL1
	LCALL SDA1
	RET

;I2C bus lines control

SDA0:	CLR SDA		;SDA 1/0
	NOP
	RET

SDA1:	SETB SDA		;SDA 0/1
	NOP
	RET

SCL0:	CLR SCL		;SCL 1/0
	NOP
	RET

SCL1:	SETB SCL		;SCL 0/1
	NOP
	RET

;------Interrupts holders------

;Timer 0 Interrupt

RTC:	PUSH ACC
	CLR TR0		;timer 0 stop
	MOV TH0,#RTCVH	;timer 0 load for 10 mS
	MOV TL0,#RTCVL
	SETB TR0		;timer start
	INC RTPC		;Real Time Program Counter INC

	MOV A,VOLTM	;Volume Control Timer check
	JZ RTC1
	DEC VOLTM
RTC1:	MOV A,MUTTM	;Mute Timer check
	JZ RTC2
	DEC MUTTM
RTC2:	MOV A,SWTM	;Switcher Timer check
	JZ RTC3
	DEC SWTM
RTC3:	MOV A,SWMTM	;Switcher Motor Timer check
	JZ RTC4
	DEC SWMTM
RTC4:	MOV A,AIDTM	;Auto Ident Timer check
	JZ RTC5
	DEC AIDTM
RTC5:	MOV A,DCLTM	;Double-Click Timer check
	JZ RTC6
	DEC DCLTM

RTC6:	POP ACC
	RETI

;INT 0 interrupt

#INCLUDE "LIBRC5.ASM"

;  RC5 program decoder is interrupt holder.
;  Input - bit SER (interrupt line), low active level.
;  Out - byte R37 (register R7 in bank 3),
;                  D0-D5 - command bits
;                  D6 - control bit of local keyboard
;                 (D6 does not change)
;                  D7 - control bit RC
;  Uses register bank 3
;  Delay values are selecting by defining
;  CLK_8MHZ, CLK_10MHZ or CLK_12MHZ in main program.
;  Main program must include:
;  SYS .EQU 000H
;  SER .EQU INT0 (or INT1)
;      .ORG 0003H ;INT0 vector (or 0013H ;INT1 vector)
;      LJMP RC5
;      INT0 (or INT1) enable
;  RCTAB .DB ... (64 byte recoding table)

;RC Recoding Table

RCTAB     .DB CODVLP	;key code 000H, key function - VOLUME PLUS
	.DB CODCDP	;key code 001H, key function - INPUT CD
	.DB CODTD1	;key code 002H, key function - INPUT DECK 1
	.DB CODTD2	;key code 003H, key function - INPUT DECK 2
	.DB CODTUN	;key code 004H, key function - INPUT TUNER
	.DB CODAUX	;key code 005H, key function - INPUT AUX
	.DB CODPHO	;key code 006H, key function - INPUT PHONO
	.DB CODMIC	;key code 007H, key function - INPUT MIC
	.DB CODDAT	;key code 008H, key function - INPUT DAT
	.DB CODAID	;key code 009H, key function - AUTO IDENT
	.DB CODVLM	;key code 00AH, key function - VOLUME MINUS
	.DB NONE		;key code 00BH, key function - none
	.DB NONE		;key code 00CH, key function - none
	.DB CODSTB	;key code 00DH, key function - STANDBY
	.DB CODSWP	;key code 00EH, key function - SWAP
	.DB CODMUT	;key code 00FH, key function - MUTE
	.DB CODTCH	;key code 010H, key function - TEACH
	.DB CODASB	;key code 011H, key function - AUTO STANDBY
	.DB NONE		;key code 012H, key function - none
	.DB NONE		;key code 013H, key function - none
	.DB NONE		;key code 014H, key function - none
	.DB NONE		;key code 015H, key function - none
	.DB NONE		;key code 016H, key function - none
	.DB NONE		;key code 017H, key function - none
	.DB NONE		;key code 018H, key function - none
	.DB NONE		;key code 019H, key function - none
	.DB NONE		;key code 01AH, key function - none
	.DB NONE		;key code 01BH, key function - none
	.DB NONE		;key code 01CH, key function - none
	.DB NONE		;key code 01DH, key function - none
	.DB NONE 		;key code 01EH, key function - none
	.DB NONE		;key code 01FH, key function - none
	.DB NONE		;key code 020H, key function - none
	.DB NONE		;key code 021H, key function - none
	.DB NONE 		;key code 022H, key function - none
	.DB NONE		;key code 023H, key function - none
	.DB NONE		;key code 024H, key function - none
	.DB NONE		;key code 025H, key function - none
	.DB CODUSW	;key code 026H, key function - UNSWAP
	.DB NONE		;key code 027H, key function - none
	.DB NONE		;key code 028H, key function - none
	.DB NONE		;key code 029H, key function - none
	.DB NONE 		;key code 02AH, key function - none
	.DB NONE		;key code 02BH, key function - none
	.DB NONE		;key code 02CH, key function - none
	.DB NONE 		;key code 02DH, key function - none
	.DB NONE		;key code 02EH, key function - none
	.DB NONE		;key code 02FH, key function - none
	.DB NONE		;key code 030H, key function - none
	.DB NONE		;key code 031H, key function - none
	.DB NONE		;key code 032H, key function - none
	.DB NONE		;key code 033H, key function - none
	.DB NONE		;key code 034H, key function - none
	.DB NONE 		;key code 035H, key function - none
	.DB NONE		;key code 036H, key function - none
	.DB NONE 		;key code 037H, key function - none
	.DB CODMOD	;key code 038H, key function - MODE
	.DB NONE		;key code 039H, key function - none
	.DB NONE		;key code 03AH, key function - none
	.DB CODEQU	;key code 03BH, key function - EQ. BYPASS
	.DB NONE 		;key code 03CH, key function - none
	.DB NONE		;key code 03DH, key function - none
	.DB NONE		;key code 03EH, key function - none
	.DB CODDIS	;key code 03FH, key function - DISPLAY

;Local Keyboard Recoding Table

KEYTAB:   .DB CODMOD	;key code 000H, key function - MODE
	.DB CODSTB	;key code 001H, key function - STANDBY
	.DB CODMUT	;key code 002H, key function - MUTE
	.DB CODEQU	;key code 003H, key function - EQ. BYPASS
	.DB NONE		;key code 008H, key function - none

; ------ Control Functions Codes ------

NONE	.EQU 000H	;key function - none
CODSTB	.EQU 001H	;key function - STANDBY
CODMUT	.EQU 002H	;key function - MUTE
CODMOD	.EQU 003H	;key function - MODE
CODEQU	.EQU 004H	;key function - EQ. BYPASS
CODDIS	.EQU 005H	;key function - DISPLAY
CODSWP	.EQU 006H	;key function - SWAP
CODAID	.EQU 007H ;key function - AUTO IDENT
CODASB	.EQU 008H	;key function - AUTO STANDBY
CODTCH	.EQU 009H	;key function - TEACH
CODUSW	.EQU 00AH	;key function - UNSWAP

CODVLP	.EQU 010H	;key function - VOLUME PLUS
CODVLM	.EQU 011H	;key function - VOLUME MINUS

CODDAT	.EQU 020H	;key function - INPUT DAT
CODCDP	.EQU 021H	;key function - INPUT CD
CODTD1	.EQU 022H	;key function - INPUT TAPE DECK 1
CODTD2	.EQU 023H	;key function - INPUT TAPE DECK 2
CODTUN	.EQU 024H	;key function - INPUT TUNER
CODAUX	.EQU 025H	;key function - INPUT AUX
CODPHO	.EQU 026H	;key function - INPUT PHONO
CODMIC	.EQU 027H	;key function - INPUT MIC

;Spectrum Analyzer Registers Addresses Table

SATAB	.DB 000H,002H,001H,000H,007H,006H,005H,004H

;log level table (step 2dB)

;LTAB	.DB 033H,040H,051H,066H,080H,0A1H,0CBH,0FFH

;log level table (step 3dB)

;LTAB	.DB 017H,020H,02DH,040H,05AH,080H,0B5H,0FFH

;log level table (step 4dB)

;LTAB	.DB 00AH,010H,019H,028H,040H,066H,0A1H,0FFH

;log level table

LTAB	.DB 014H,01DH,028H,039H,051H,072H,0A1H,0FFH

	.END
